from pathlib import Path
from typing import Any, Dict, Iterator, Optional

import yaml

from servicefoundry.lib.clients.service_foundry_client import (
    ServiceFoundryServiceClient,
)
from servicefoundry.lib.model.entity import ApplyResult, Manifest
from servicefoundry.pydantic_v1 import ValidationError


def _apply_manifest(
    manifest: Dict[str, Any],
    client: Optional[ServiceFoundryServiceClient] = None,
    filename: Optional[str] = None,
    index: Optional[int] = None,
) -> ApplyResult:
    client = client or ServiceFoundryServiceClient()

    file_metadata = ""
    if index is not None:
        file_metadata += f" at index {index}"
    if filename:
        file_metadata += f" from file {filename}"

    try:
        manifest = Manifest.parse_obj(manifest)
    except ValidationError as ex:
        return ApplyResult(
            success=False,
            message=f"Failed to apply manifest{file_metadata}. Error: {ex}",
        )

    try:
        client.apply(manifest.dict())
        return ApplyResult(
            success=True,
            message=f"Successfully configured manifest {manifest.name} of type {manifest.type}",
        )
    except Exception as ex:
        return ApplyResult(
            success=False,
            message=f"Failed to apply manifest {manifest.name} of type {manifest.type}. Error: {ex}",
        )


def apply_manifest(
    manifest: Dict[str, Any],
    client: Optional[ServiceFoundryServiceClient] = None,
) -> ApplyResult:
    return _apply_manifest(manifest=manifest, client=client)


def apply_manifest_file(
    filepath: str,
    client: Optional[ServiceFoundryServiceClient] = None,
) -> Iterator[ApplyResult]:
    client = client or ServiceFoundryServiceClient()
    filename = Path(filepath).name
    try:
        with open(filepath, "r") as f:
            manifests_it = list(yaml.safe_load_all(f))
    except Exception as ex:
        yield ApplyResult(
            success=False,
            message=f"Failed to read file {filepath} as a valid YAML file. Error: {ex}",
        )
    else:
        for index, manifest in enumerate(manifests_it):
            if not isinstance(manifest, dict):
                yield ApplyResult(
                    success=False,
                    message=f"Failed to apply manifest at index {index} from file {filename}. Error: A manifest must be a dict, got type {type(manifest)}",
                )
                continue

            yield _apply_manifest(
                manifest=manifest, client=client, filename=filename, index=index
            )



import os
import sys
from typing import List, Tuple

import rich_click as click

from servicefoundry.cli.console import console
from servicefoundry.cli.const import GROUP_CLS
from servicefoundry.cli.util import handle_exception_wrapper
from servicefoundry.lib.clients.service_foundry_client import (
    ServiceFoundryServiceClient,
)
from servicefoundry.lib.dao import apply as apply_lib
from servicefoundry.lib.messages import PROMPT_APPLYING_MANIFEST
from servicefoundry.lib.model.entity import ApplyResult


@click.group(
    name="apply",
    cls=GROUP_CLS,
    invoke_without_command=True,
    help="Create resources by appling manifest locally from Truefoundry spec",
    context_settings=dict(ignore_unknown_options=True, allow_interspersed_args=True),
)
@click.option(
    "-f",
    "--file",
    "files",
    type=click.Path(exists=True, dir_okay=False, resolve_path=True),
    help="Path to yaml manifest file (You can apply multiple files at once by providing multiple -f options)",
    show_default=True,
    required=True,
    multiple=True,
)
@handle_exception_wrapper
def apply_command(files: Tuple[str, ...]):
    apply_results: List[ApplyResult] = []
    client = ServiceFoundryServiceClient()
    for file in files:
        with console.status(PROMPT_APPLYING_MANIFEST.format(file), spinner="dots"):
            for apply_result in apply_lib.apply_manifest_file(file, client):
                if apply_result.success:
                    console.print(f"[green]\u2714 {apply_result.message}[/]")
                else:
                    console.print(f"[red]\u2718 {apply_result.message}[/]")

                apply_results.append(apply_result)

    if not all(apply_result.success for apply_result in apply_results):
        raise Exception("Failed to apply one or more manifests")


def get_apply_command():
    return apply_command
